"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useVirtualNavigationTree = void 0;
const react_1 = require("react");
const compass_components_1 = require("@mongodb-js/compass-components");
function isTreeItem(item) {
    return item.type !== 'placeholder';
}
function isExpandable(item) {
    return typeof item.isExpanded !== 'undefined';
}
function findNext(itemIndex, items, fn, shift = 0) {
    for (let i = itemIndex + 1, len = items.length; i < len; i++) {
        const idx = (i + shift) % len;
        const item = items[idx];
        if (isTreeItem(item) && (!fn || fn(item))) {
            return item;
        }
    }
    return null;
}
function findPrev(itemIndex, items, fn, shift = 0) {
    const len = items.length;
    for (let i = itemIndex - 1; i >= 0; i--) {
        const idx = (i + shift) % len;
        const item = items[idx];
        if (isTreeItem(item) && (!fn || fn(item))) {
            return item;
        }
    }
    return null;
}
function findFirstItem(items) {
    return findNext(-1, items);
}
function findLastItem(items) {
    return findPrev(items.length, items);
}
function findParentItem(currentItem, currentItemIndex, items) {
    return findPrev(currentItemIndex, items, (item) => item.level === currentItem.level - 1);
}
function findSiblings(currentItem, currentItemIndex, items) {
    const result = [currentItem];
    for (let i = currentItemIndex - 1; i >= 0; i--) {
        const item = items[i];
        if (isTreeItem(item)) {
            if (item.level === currentItem.level) {
                result.push(item);
            }
            if (item.level === currentItem.level - 1) {
                break;
            }
        }
    }
    for (let i = currentItemIndex + 1, len = items.length; i < len; i++) {
        const item = items[i];
        if (isTreeItem(item)) {
            if (item.level === currentItem.level) {
                result.push(item);
            }
            if (item.level === currentItem.level - 1) {
                break;
            }
        }
    }
    return result;
}
function useVirtualNavigationTree({ items, activeItemId, onExpandedChange, onFocusMove = () => {
}, }) {
    const rootRef = (0, react_1.useRef)(null);
    const activeId = activeItemId || findFirstItem(items)?.id;
    const [currentTabbable, setCurrentTabbable] = (0, react_1.useState)(activeId);
    const [tabIndex, setTabIndex] = (0, react_1.useState)(0);
    const [focusProps, focusState] = (0, compass_components_1.useFocusState)();
    const focusItemById = (0, react_1.useCallback)((id) => {
        rootRef.current
            ?.querySelector(`[role="treeitem"][data-id="${id}"]`)
            ?.focus();
    }, [rootRef]);
    (0, react_1.useEffect)(() => {
        if (focusState === compass_components_1.FocusState.FocusVisible ||
            focusState === compass_components_1.FocusState.Focus) {
            setTabIndex(-1);
            const item = findNext(-1, items, (item) => item.id === currentTabbable);
            if (item) {
                onFocusMove(item);
                const cancel = (0, compass_components_1.rafraf)(() => {
                    focusItemById(item.id);
                });
                return cancel;
            }
        }
        if (focusState === compass_components_1.FocusState.FocusWithin ||
            focusState === compass_components_1.FocusState.FocusWithinVisible) {
            setTabIndex(-1);
        }
        if (focusState === compass_components_1.FocusState.NoFocus) {
            setTabIndex(0);
        }
    }, [currentTabbable, focusItemById, focusState, items, onFocusMove]);
    (0, react_1.useEffect)(() => {
        setCurrentTabbable(activeId);
    }, [activeId]);
    const onKeyDown = (0, react_1.useCallback)((evt) => {
        const treeItem = evt.target;
        if (treeItem.getAttribute('role') !== 'treeitem') {
            return;
        }
        const currentItemId = treeItem.dataset.id;
        if (!currentItemId) {
            throw new Error('treeitems should have data-id attribute provided to work correctly');
        }
        const currentItem = findNext(-1, items, (item) => item.id === currentItemId);
        if (!currentItem) {
            throw new Error('key event caught on an item that is not part of the items list');
        }
        const currentItemIndex = items.indexOf(currentItem);
        let nextItem = null;
        if (evt.key === 'Home') {
            evt.stopPropagation();
            nextItem = findFirstItem(items);
        }
        if (evt.key === 'End') {
            evt.stopPropagation();
            nextItem = findLastItem(items);
        }
        if (evt.key === 'ArrowDown') {
            evt.preventDefault();
            evt.stopPropagation();
            nextItem = findNext(currentItemIndex, items);
        }
        if (evt.key === 'ArrowUp') {
            evt.preventDefault();
            evt.stopPropagation();
            nextItem = findPrev(currentItemIndex, items);
        }
        if (evt.key === 'ArrowRight') {
            evt.stopPropagation();
            if (isExpandable(currentItem) && currentItem.isExpanded === false) {
                onExpandedChange(currentItem, true);
            }
            if (isExpandable(currentItem) && currentItem.isExpanded === true) {
                const maybeNextItem = findNext(currentItemIndex, items);
                if (maybeNextItem?.level === currentItem.level + 1) {
                    nextItem = maybeNextItem;
                }
            }
        }
        if (evt.key === 'ArrowLeft') {
            evt.stopPropagation();
            if (isExpandable(currentItem) && currentItem.isExpanded === true) {
                onExpandedChange(currentItem, false);
            }
            else {
                nextItem = findParentItem(currentItem, currentItemIndex, items);
            }
        }
        if (evt.key === '*') {
            evt.stopPropagation();
            const siblings = findSiblings(currentItem, currentItemIndex, items);
            for (const item of siblings) {
                if (isExpandable(currentItem) && item.isExpanded === false) {
                    onExpandedChange(item, true);
                }
            }
        }
        if (/^\p{Letter}$/u.test(evt.key)) {
            evt.stopPropagation();
            const letter = evt.key.toLocaleLowerCase();
            nextItem = findNext(0, items, (item) => item.name.toLocaleLowerCase().startsWith(letter), currentItemIndex);
        }
        if (nextItem && nextItem !== currentItem) {
            const id = nextItem.id;
            onFocusMove(nextItem);
            setCurrentTabbable(id);
            requestAnimationFrame(() => {
                focusItemById(id);
            });
        }
    }, [onExpandedChange, onFocusMove, focusItemById, items]);
    const rootProps = {
        ref: rootRef,
        tabIndex,
        onKeyDown,
        ...focusProps,
    };
    return [rootProps, currentTabbable];
}
exports.useVirtualNavigationTree = useVirtualNavigationTree;
//# sourceMappingURL=use-virtual-navigation-tree.js.map